%  AMB-Programs (Version: 0.1):  Approximation and Modeling with B-Splines
%
% PROGRAMS
%
% Chapter 1: Polynomials
%
%   Programs:
%   BERNSTEIN: Bernstein polynomials
%   HERMITE: evaluation of a Hermite spline
%
%   Interactive Demos: 
%   DEMO_WEIERSTRASS_APPROXIMATION
%   DEMO_HERMITE_INTERPOLATION 
%
% Chapter 2: Bezier Curves
%
%   Programs:
%   BEZIER_SUBDIVISION: subdivision of a Bezier curve
%   BEZIER2POLY: conversion from Bezier to Taylor form
%   CASTELJAU: evaluation of a Bezier curve
%   HERMITE_GEOMETRIC: planar geometric Hermite interpolant
%
%   Interactive Demos: 
%   DEMO_BEZIER_CURVE 
%   DEMO_GEOMETRIC_HERMITE_INTERPOLATION 
%   DEMO_CASTELJAU_ALGORITHM 
%
% Chapter 3: Rational Bezier Curves
%
%   Programs:
%   RATIONAL_BEZIER_CURVE: evaluation of a rational Bezier curve 
%   RATIONAL_BEZIER2MAT: Matrix representation of a conic section 
%
%   Interactive Demos: 
%   DEMO_RATIONAL_BEZIER_CURVE
%   DEMO_CONIC_SECTION 
%
% Chapter 4: B-Splines
%
%   Programs:
%   BSPLINE: evaluation of a B-spline  
%   SPLINE_EVALUATION: evaluation of a spline function 
%   SPLINE_DIFFERENTIATION: derivative of a spline 
%
%   Interactive Demos: 
%   DEMO_BSPLINE
%   DEMO_SPLINE 
%
% Chapter 5: Approximation
%
%   Programs:
%   QUASI_INTERPOLATION: weights for quasi-interpolation
%   SPLINE_INTERPOLATION: spline interpolation   
%   SPLINE_INTERPOLATION_CUBIC: cubic spline interpolation   
%   SMOOTHING_SPLINE: cubic smoothing spline
%
%   Interactive Demos: 
%   DEMO_STANDARD_PROJECTOR
%   DEMO_SMOOTHING_SPLINE 
%
% Chapter 6: Spline Curves
%
%   Programs:
%   SPLINE_CURVE: evaluation of a spline curve
%   CLOSED_SPLINE_CURVE: evaluation of a closed spline curve
%   NURBS: evaluation of a rational spline curve
%   KNOT_INSERTION: knot insertion for splines
%   UNIFORM_SUBDIVISION: midpoint subdivision
%
%   Interactive Demos: 
%   DEMO_SPLINE_CURVE
%   DEMO_UNIFORM_SUBDIVISION 
%
% Chapter 7: Multivariate Splines
%
%   Programs:
%   BSPLINE_MULTIVARIATE: evaluation of a multivariate B-spline
%   SPLINE_EVALUATION_BIVARIATE: evaluation of a bivariate spline function
%
%   Interactive Demos: 
%   DEMO_BSPLINE_BIVARIATE
%   DEMO_SPLINE_BIVARIATE
%
%
% Chapter 8: Surfaces and Solids
%
%   Programs:
%   BEZIER_PATCH: evaluation of a Bezier patch
%   SPLINE_SURFACE: evaluation of a spline surface
%   SPLINE_SOLID: evaluation of a spline solid
%
%   Interactive Demos: 
%   DEMO_BEZIER_PATCH
%   DEMO_CATMULL_CLARK
%
%
% Chapter 9: Finite Elements
%
%    see the supplementary material to the SIAM book 
%    "Finite Element Methods with B-Splines" 
%    (http://www.siam.org/books/fr26/) for programs and demos


% authors/copyright 2014: Klaus Hoellig and Joerg Hoerner
% 
% The programs supplement the SIAM book 
% "Approximation an Modeling with  B-Splines" (OT 132, 2013)
% and are intended for educational use only.
% See TermsofUse.txt for details.
